@@ -4,7 +4,7 @@
  * GChemPaint library
  * bond.cc 
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -35,18 +35,20 @@
 #include "libgcpcanvas/gcp-canvas-line.h"
 #include "libgcpcanvas/gcp-canvas-bpath.h"
 #include "libgcpcanvas/gcp-canvas-polygon.h"
-#include <cmath>
+#include <math.h>
 
 gcpBond::gcpBond(): Bond()
 {
 	m_CoordsCalc = false;
 	m_type = NormalBondType;
+	m_level = 0;
 }
 
 gcpBond::gcpBond(gcpAtom* first, gcpAtom* last, unsigned char order): Bond(first, last, order)
 {
 	m_CoordsCalc = false;
 	m_type = NormalBondType;
+	m_level = 0;
 }
 
 gcpBond::~gcpBond()
@@ -223,40 +225,55 @@ Object* gcpBond::GetAtomAt(double x, double y, double z)
 	return NULL;
 }
 
-bool gcpBond::SaveNode (xmlDocPtr xml, xmlNodePtr node)
+bool gcpBond::SaveNode(xmlDocPtr xml, xmlNodePtr node)
 {
-	switch (m_type) {
+	switch(m_type) {
 	case UpBondType:
-		xmlNewProp (node, (xmlChar*) "type", (xmlChar*) "up");
+		xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"up");
 		break;
 	case DownBondType:
-		xmlNewProp (node, (xmlChar*) "type", (xmlChar*) "down");
+		xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"down");
+		break;
+	case ForeBondType:
+		xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"fore");
 		break;
 	case UndeterminedBondType:
-		xmlNewProp (node, (xmlChar*) "type", (xmlChar*) "undetermined");
+		xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"undetermined");
 		break;
 	default:
 		break;
 	}
+	if (m_level != 0) {
+		char *buf = g_strdup_printf ("%d", m_level);
+		xmlNewProp(node, (xmlChar*)"level", (xmlChar*) buf);
+		g_free (buf);
+	}
 	return true;
 }
 
 bool gcpBond::LoadNode (xmlNodePtr node)
 {
-	char* tmp;
-	tmp = (char*) xmlGetProp (node, (xmlChar*) "type");
-	if (!tmp)
+	char* buf;
+	buf = (char*) xmlGetProp (node, (xmlChar*) "type");
+	if (!buf)
 		SetType (NormalBondType);
-	else if (!strcmp (tmp, "up"))
+	else if (!strcmp (buf, "up"))
 		SetType (UpBondType);
-	else if (!strcmp (tmp, "down"))
+	else if (!strcmp (buf, "down"))
 		SetType (DownBondType);
-	else if (!strcmp (tmp, "undetermined"))
+	else if (!strcmp (buf, "fore"))
+		SetType (ForeBondType);
+	else if (!strcmp (buf, "undetermined"))
 		SetType (UndeterminedBondType);
 	else
 		SetType (NormalBondType);
-	if (tmp)
-		xmlFree (tmp);
+	if (buf)
+		xmlFree (buf);
+	buf = (char*) xmlGetProp (node, (xmlChar*) "level");
+	if (buf) {
+		m_level = atoi (buf);
+		xmlFree (buf);
+	}
 	return true;
 }
 
@@ -351,13 +368,22 @@ void gcpBond::SetSelected(GtkWidget* w, int state)
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
 	GnomeCanvasGroup* group = pData->Items[this];
 	gchar* color;
-	switch (state)
-	{	
-		case SelStateUnselected: color = Color; break;
-		case SelStateSelected: color = SelectColor; break;
-		case SelStateUpdating: color = AddColor; break;
-		case SelStateErasing: color = DeleteColor; break;
-		default: color = Color; break;
+	switch (state) {	
+	case SelStateUnselected:
+		color = Color;
+		break;
+	case SelStateSelected:
+		color = SelectColor;
+		break;
+	case SelStateUpdating:
+		color = AddColor;
+		break;
+	case SelStateErasing:
+		color = DeleteColor;
+		break;
+	default:
+		color = Color;
+		break;
 	}
 	void *obj = g_object_get_data(G_OBJECT(group), "path");
 	switch (GetType())
@@ -366,6 +392,7 @@ void gcpBond::SetSelected(GtkWidget* w, int state)
 		case UndeterminedBondType:
 			g_object_set(obj, "outline_color", color, NULL);
 			break;
+		case ForeBondType:
 		case UpBondType:
 		case DownBondType:
 			g_object_set(obj, "fill_color", color, NULL);
@@ -388,7 +415,48 @@ void gcpBond::Add(GtkWidget* w)
 	group = GNOME_CANVAS_GROUP(gnome_canvas_item_new(pData->Group, gnome_canvas_group_ext_get_type(), NULL));
 	g_signal_connect(G_OBJECT(group), "event", G_CALLBACK(on_event), w);
 	g_object_set_data(G_OBJECT(group), "object", this);
-	GnomeCanvasPathDef *path = BuildPathDef (pData);
+	bool result = false;
+	if (m_Crossing.size () > 0) {
+		map<gcpBond*, gcpBondCrossing>::iterator i, iend = m_Crossing.end ();
+		for (i = m_Crossing.begin (); i != iend; i++)
+			if ((result |= (*i).second.is_before))
+				break;
+	}
+	GnomeCanvasPathDef *path = NULL;
+	if (result) {
+		path = BuildCrossingPathDef (pData);
+		if (path) {
+			GnomeCanvasItem *item = (m_type == NormalBondType || m_type == UndeterminedBondType)?
+				gnome_canvas_item_new (
+					group,
+					gnome_canvas_bpath_ext_get_type (),
+					"bpath", path,
+					"outline_color", "white",
+					"width_units", pData->BondWidth * 3.,
+					NULL):
+				gnome_canvas_item_new (
+					group,
+					gnome_canvas_bpath_ext_get_type(),
+					"bpath", path,
+					"fill_color", "white",
+					"width_units", 0.,
+					NULL);
+			g_object_set_data (G_OBJECT (group), "back", item);
+			g_object_set_data (G_OBJECT (item), "object", this);
+			g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
+			/* now bring to front, FIXME: not secure ! */
+			gnome_canvas_item_lower_to_bottom (item);
+			gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (group));
+			Atom *pAtom = GetAtom (0);
+			if (pAtom->GetZ () != 6)
+				gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[pAtom]));
+			pAtom = GetAtom (1);
+			if (pAtom->GetZ () != 6)
+				gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[pAtom]));
+			gnome_canvas_path_def_unref (path);
+		}
+	}
+	path = BuildPathDef (pData);
 	switch (GetType())
 	{
 		case NormalBondType:
@@ -401,6 +469,7 @@ void gcpBond::Add(GtkWidget* w)
 									"width_units", pData->BondWidth,
 									NULL);
 			break;
+		case ForeBondType:
 		case UpBondType:
 		case DownBondType:
 			item = gnome_canvas_item_new(
@@ -533,6 +602,17 @@ GnomeCanvasPathDef* gcpBond::BuildPathDef (gcpWidgetData* pData)
 			}
 			break;
 		}
+		case ForeBondType:
+			GetLine2DCoords(1, &x1, &y1, &x2, &y2);
+			x = sqrt(square(x2 - x1) + square(y2 - y1));
+			dx = (y1 - y2) / x * pData->StereoBondWidth / 2;
+			dy = (x2 - x1) / x * pData->StereoBondWidth / 2;
+			gnome_canvas_path_def_moveto (path, x1 * pData->ZoomFactor + dx, y1 * pData->ZoomFactor + dy);
+			gnome_canvas_path_def_lineto (path, x2 * pData->ZoomFactor + dx, y2 * pData->ZoomFactor + dy);
+			gnome_canvas_path_def_lineto (path, x2 * pData->ZoomFactor -dx, y2 * pData->ZoomFactor -dy);
+			gnome_canvas_path_def_lineto (path, x1 * pData->ZoomFactor -dx, y1 * pData->ZoomFactor -dy);
+			gnome_canvas_path_def_closepath (path);
+			break;
 		case UndeterminedBondType:
 		{
 			GetLine2DCoords(1, &x1, &y1, &x2, &y2);
@@ -571,13 +651,112 @@ GnomeCanvasPathDef* gcpBond::BuildPathDef (gcpWidgetData* pData)
 	return path;
 }
 
+GnomeCanvasPathDef *gcpBond::BuildCrossingPathDef (gcpWidgetData* pData)
+{
+	GnomeCanvasPathDef* path = NULL;
+	double x1, y1, x2, y2, x, dx, dy;
+	int i;
+	switch (GetType ()) {
+	case NormalBondType:
+		i = 1;
+		path = gnome_canvas_path_def_new ();
+		while (GetLine2DCoords (i++, &x1, &y1, &x2, &y2)) {
+			dx = (x2 - x1) / 10.;
+			dy = (y2 - y1) / 10.;
+			x1 += dx;
+			x2 -= dx;
+			y1 += dy;
+			y2 -= dy;
+			gnome_canvas_path_def_moveto (path, x1 * pData->ZoomFactor, y1 * pData->ZoomFactor);
+			gnome_canvas_path_def_lineto (path, x2 * pData->ZoomFactor, y2 * pData->ZoomFactor);
+		}
+		break;
+	case UpBondType:
+		break;
+	case DownBondType:
+		break;
+	case ForeBondType:
+		path = gnome_canvas_path_def_new ();
+		GetLine2DCoords(1, &x1, &y1, &x2, &y2);
+		dx = (x2 - x1) / 10.;
+		dy = (y2 - y1) / 10.;
+		x1 += dx;
+		x2 -= dx;
+		y1 += dy;
+		y2 -= dy;
+		x = sqrt(square(x2 - x1) + square(y2 - y1));
+		dx = (y1 - y2) / x * pData->StereoBondWidth / 2;
+		dy = (x2 - x1) / x * pData->StereoBondWidth / 2;
+		dx += (dx > 0.)? 1.: -1.;
+		dy += (dy > 0.)? 1.: -1.;
+		gnome_canvas_path_def_moveto (path, x1 * pData->ZoomFactor + dx, y1 * pData->ZoomFactor + dy);
+		gnome_canvas_path_def_lineto (path, x2 * pData->ZoomFactor + dx, y2 * pData->ZoomFactor + dy);
+		gnome_canvas_path_def_lineto (path, x2 * pData->ZoomFactor -dx, y2 * pData->ZoomFactor -dy);
+		gnome_canvas_path_def_lineto (path, x1 * pData->ZoomFactor -dx, y1 * pData->ZoomFactor -dy);
+		gnome_canvas_path_def_closepath (path);
+		break;
+	case UndeterminedBondType:
+		break;
+	}
+	return path;
+}
+
 void gcpBond::Update(GtkWidget* w)
 {
 	if (!w) return;
 	gcpWidgetData* pData = (gcpWidgetData*) g_object_get_data(G_OBJECT(w), "data");
-	GnomeCanvasPathDef* path = BuildPathDef (pData);
+	bool result = false;
+	if (m_Crossing.size () > 0) {
+		map<gcpBond*, gcpBondCrossing>::iterator i, iend = m_Crossing.end ();
+		for (i = m_Crossing.begin (); i != iend; i++)
+			if ((result |= (*i).second.is_before))
+				break;
+	}
+	GnomeCanvasPathDef* path;
+	void * obj;
 	GnomeCanvasGroup *group = pData->Items[this];
-	void *obj = g_object_get_data(G_OBJECT(group), "path");
+	obj = g_object_get_data(G_OBJECT(group), "back");
+	if (result) {
+		path = BuildCrossingPathDef (pData);
+		if (path) {
+			if (obj)
+				g_object_set (obj, "bpath", path, NULL);
+			else {
+				GnomeCanvasItem *item = (m_type == NormalBondType || m_type == UndeterminedBondType)?
+					gnome_canvas_item_new (
+						group,
+						gnome_canvas_bpath_ext_get_type (),
+						"bpath", path,
+						"outline_color", "white",
+						"width_units", pData->BondWidth * 3.,
+						NULL):
+					gnome_canvas_item_new (
+						group,
+						gnome_canvas_bpath_ext_get_type(),
+						"bpath", path,
+						"fill_color", "white",
+						"width_units", 0.,
+						NULL);
+				g_object_set_data (G_OBJECT (group), "back", item);
+				g_object_set_data (G_OBJECT (item), "object", this);
+				g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
+				/* now bring to front, FIXME: not secure ! */
+				gnome_canvas_item_lower_to_bottom (item);
+				gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (group));
+				Atom *pAtom = GetAtom (0);
+				if (pAtom->GetZ () != 6)
+					gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[pAtom]));
+				pAtom = GetAtom (1);
+				if (pAtom->GetZ () != 6)
+					gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[pAtom]));
+			}
+			gnome_canvas_path_def_unref (path);
+		}
+	} else if (obj) {
+		g_object_set_data (G_OBJECT(group), "back", NULL);
+	}
+	path = BuildPathDef (pData);
+	obj = g_object_get_data(G_OBJECT(group), "path");
 	g_object_set (obj, "bpath", path, NULL);
 	gnome_canvas_path_def_unref (path);
 }
@@ -589,3 +768,129 @@ double gcpBond::GetYAlign ()
 	m_End->GetCoords(&x2, &y2);
 	return (y1 + y2) / 2;
 }
+
+bool gcpBond::IsCrossing (gcpBond *pBond)
+{
+	double a1, a2, b1, b2, c1, c2, d, d1, d2;
+	double x0, x1, x2, x3, y0, y1, y2, y3, z0, z1, z2, z3;
+	if (m_Begin == pBond->m_Begin || m_Begin == pBond->m_End ||
+			m_End == pBond->m_Begin || m_End == pBond->m_End)
+		return false;
+	m_Begin->GetCoords(&x0, &y0, &z0);
+	m_End->GetCoords(&x1, &y1, &z1);
+	pBond->m_Begin->GetCoords(&x2, &y2, &z2);
+	pBond->m_End->GetCoords(&x3, &y3, &z3);
+	a1 = x1 - x0;
+	a2 = y1 - y0;
+	b1 = x2 - x3;
+	b2 = y2 - y3;
+	d = a1 * b2 - a2 * b1;
+	if (d == 0.)
+		return false;
+	c1 = x2 - x0;
+	c2 = y2 - y0;
+	d1 = c1 * b2 - c2 * b1;
+	d2 = c2 * a1 - c1 * a2;
+	a1 = d1 / d;
+	a2 = d2 / d;
+	if ((a1 <= 0.) || (a1 >= 1.))
+		return false;
+	if ((a2 > 0.) && (a2 < 1.)) {
+		double z = z0 + a1 * (z1 - z0), z_ = z2 + a2 * (z3 - z2);
+		bool is_before = z > z_ || m_level > pBond->m_level;
+		if (z == z_ && m_level == pBond->m_level) {
+			if (m_type == ForeBondType && pBond->m_type != ForeBondType) {
+				is_before = true;
+				pBond->m_level -= 1;
+			} else {
+				is_before = false;
+				pBond->m_level += 1;
+			}
+		}
+		m_Crossing[pBond].a = a1;
+		m_Crossing[pBond].is_before = is_before;
+		pBond->m_Crossing[this].a = a2;
+		pBond->m_Crossing[this].is_before = !is_before;
+		return true;
+	} else
+		return false;
+}
+
+static void on_bring_to_front (gcpBond *pBond)
+{
+	pBond->BringToFront ();
+}
+
+static void on_move_to_back (gcpBond *pBond)
+{
+	pBond->MoveToBack ();
+}
+
+bool gcpBond::BuildContextualMenu (GtkUIManager *UIManager, Object *object)
+{
+	if (m_Crossing.size () == 0)
+		return GetParent ()->BuildContextualMenu (UIManager, object);;
+	bool is_before = false, is_after = false;
+	if (m_Crossing.size () > 0) {
+		map<gcpBond*, gcpBondCrossing>::iterator i, iend = m_Crossing.end ();
+		for (i = m_Crossing.begin (); i != iend; i++) {
+			if (m_level == (*i).first->m_level || m_type != (*i).first->m_type)
+				continue;
+			if ((*i).second.is_before)
+				is_before = true;
+			else
+				is_after = true;
+		}
+	}
+	if (!(is_before || is_after))
+		return GetParent ()->BuildContextualMenu (UIManager, object);;
+	GtkActionGroup *group = gtk_action_group_new ("bond");
+	GtkAction *action;
+	action = gtk_action_new ("Bond", _("Bond"), NULL, NULL);
+	gtk_action_group_add_action (group, action);
+	if (is_before) {
+		action = gtk_action_new ("MoveBack", _("Move to back"), NULL, NULL);
+		g_signal_connect_swapped (action, "activate", G_CALLBACK (on_move_to_back), this);
+		gtk_action_group_add_action (group, action);
+		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Bond'><menuitem action='MoveBack'/></menu></popup></ui>", -1, NULL);
+	}
+	if (is_after) {
+		action = gtk_action_new ("BringFront", _("Bring to front"), NULL, NULL);
+		g_signal_connect_swapped (action, "activate", G_CALLBACK (on_bring_to_front), this);
+		gtk_action_group_add_action (group, action);
+		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Bond'><menuitem action='BringFront'/></menu></popup></ui>", -1, NULL);
+	}
+	gtk_ui_manager_insert_action_group (UIManager, group, 0);
+	GetParent ()->BuildContextualMenu (UIManager, object);
+	return true;
+}
+
+void gcpBond::MoveToBack ()
+{
+	gcpView *pView = reinterpret_cast<gcpDocument*> (GetDocument ())->GetView ();
+	map<gcpBond*, gcpBondCrossing>::iterator i, iend = m_Crossing.end ();
+	for (i = m_Crossing.begin (); i != iend; i++) {
+		if (m_level > (*i).first->m_level && m_type == (*i).first->m_type) {
+			m_level = (*i).first->m_level - 1;
+			(*i).second.is_before = false;
+			(*i).first->m_Crossing[this].is_before = true;
+			pView->Update ((*i).first);
+		}
+	}
+	pView->Update (this);
+}
+
+void gcpBond::BringToFront ()
+{
+	gcpView *pView = reinterpret_cast<gcpDocument*> (GetDocument ())->GetView ();
+	map<gcpBond*, gcpBondCrossing>::iterator i, iend = m_Crossing.end ();
+	for (i = m_Crossing.begin (); i != iend; i++) {
+		if (m_level < (*i).first->m_level && m_type == (*i).first->m_type) {
+			m_level = (*i).first->m_level + 1;
+			(*i).second.is_before = true;
+			(*i).first->m_Crossing[this].is_before = false;
+			pView->Update ((*i).first);
+		}
+	}
+	pView->Update (this);
+}
