@@ -44,7 +44,7 @@ gcpReaction::~gcpReaction ()
 	gcpReactionArrow *arrow;
 	gcpDocument *pDoc = reinterpret_cast<gcpDocument *> (GetDocument ());
 	gcpOperation *pOp = pDoc->GetCurrentOperation ();
-	while ((pObj = GetFirstChild (i))) {
+	while (pObj = GetFirstChild (i)) {
 		if (pObj->GetType () == ReactionArrowType) {
 			arrow = reinterpret_cast<gcpReactionArrow*> (pObj);
 			arrow->SetStartStep (NULL);
@@ -80,7 +80,7 @@ bool gcpReaction::Build (list<Object*>& Children) throw (invalid_argument)
 	list<Object*> Others;
 	map<double, Object*> Left, Right;
 	double x0, y0, x1, y1, x, y, xpos, ypos, l;
-	bool horiz = true;
+	bool horiz = true;;
 	ArtDRect *rect, srect;
 	for (i = Children.begin (); i != end; i++) {
 		// It might be better to use the objects coordinates there
@@ -391,9 +391,11 @@ double gcpReaction::GetYAlign ()
 	Object *pObj;
 	pObj = GetFirstChild (i);
 	double y = DBL_MAX, new_y;
-	while (pObj)
+	while (pObj) {
 		if (pObj->GetType () == ReactionStepType)
 			if ((new_y = pObj->GetYAlign ()) < y)
-				y = new_y;			
+				y = new_y;	
+		pObj = GetNextChild (i);
+	}			
 	return y;
 }
