@@ -40,6 +40,8 @@
 #include <locale.h>
 #include <fstream>
 
+extern bool new_ui;
+
 /*
 Derivation of a new widget from gnome_canvas with an event for updating canvas size
 */
@@ -149,9 +151,9 @@ static bool on_destroy(GtkWidget *widget, gcpView * pView)
 	return true;
 }
 
-static bool on_size(GtkWidget *widget, GtkAllocation *alloc, gcpView * pView)
+static bool on_size (GtkWidget *widget, GtkAllocation *alloc, gcpView * pView)
 {
-	pView->OnSize(alloc->width, alloc->height);
+	pView->OnSize (widget, alloc->width, alloc->height);
 	return true;
 }
 
@@ -208,7 +210,7 @@ gcpView::gcpView (gcpDocument *pDoc, bool Embedded)
 	m_width = 400;
 	m_height = 300;
 	m_ActiveRichText = NULL;
-	if (!(m_bEmbedded = Embedded))
+	if (!(m_bEmbedded = Embedded) && !new_ui)
 		CreateNewWidget ();
 	m_UIManager = gtk_ui_manager_new ();
 	m_Dragging = false;
@@ -229,8 +231,10 @@ gcpView::~gcpView ()
 
 bool gcpView::OnEvent(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 {
-	gcpTool* pActiveTool = m_pDoc->GetApplication()->GetActiveTool();
-	if ((!m_pDoc->GetEditable()) || (!pActiveTool)) return true;
+	gcpApplication *App = m_pDoc->GetApplication ();
+	gcpTool* pActiveTool = App? App->GetActiveTool (): NULL;
+	if ((!m_pDoc->GetEditable ()) || (!pActiveTool))
+		return true;
 	if (item == (GnomeCanvasItem*)m_ActiveRichText)
 	{
 		GnomeCanvasItemClass* klass = GNOME_CANVAS_ITEM_CLASS(((GTypeInstance*)item)->g_class);
@@ -312,7 +316,7 @@ bool gcpView::OnEvent(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 		pActiveTool->OnDrag(x, y, event->button.state);
 		return true;
 	case GDK_BUTTON_RELEASE:
-		switch (event->button.button)
+	switch (event->button.button)
 		{
 		case 1:
 			if (!m_Dragging) break;
@@ -320,15 +324,12 @@ bool gcpView::OnEvent(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 			pActiveTool->OnRelease(x, y, event->button.state);
 			m_pDoc->GetApplication()->ClearStatus();
 			return true;
-		default:
-			break;
 		}
 		break;
 	default:
 		break;
 	}
 	return false;
-	
 }
 
 void gcpView::AddObject(Object* pObject)
@@ -354,7 +355,6 @@ GtkWidget* gcpView::CreateNewWidget()
 		m_pData->View = this;
 		gnome_canvas_set_pixels_per_unit(GNOME_CANVAS(m_pWidget), 1);
 		gnome_canvas_set_scroll_region(GNOME_CANVAS(m_pWidget), 0, 0, m_width, m_height);
-//		gtk_widget_set_size_request(m_pWidget, m_width, m_height);//FIXME: probably necessary when embedded
 		m_pData->ZoomFactor = DefaultZoomFactor;
 		m_pData->Padding = m_DefaultPadding * DefaultZoomFactor;
 		m_pData->BondWidth = m_DefaultBondWidth * DefaultZoomFactor;
@@ -370,6 +370,7 @@ GtkWidget* gcpView::CreateNewWidget()
 		m_pData->StereoBondWidth = m_DefaultStereoBondWidth * DefaultZoomFactor;
 		m_pData->ObjectPadding = m_DefaultObjectPadding * DefaultZoomFactor;
 		m_pData->SignPadding = m_DefaultSignPadding * DefaultZoomFactor;
+		m_pData->Zoom = 1.0;
 		m_pData->Background = gnome_canvas_item_new(
 									gnome_canvas_root(GNOME_CANVAS(m_pWidget)),
 									gnome_canvas_rect_get_type(),
@@ -383,9 +384,11 @@ GtkWidget* gcpView::CreateNewWidget()
 									gnome_canvas_root(GNOME_CANVAS(m_pWidget)),
 									gnome_canvas_group_ext_get_type (),
 									NULL));
-		g_signal_connect(G_OBJECT(m_pData->Background), "event", G_CALLBACK(on_event), m_pWidget);
+		if (m_pDoc->GetEditable ())
+			g_signal_connect(G_OBJECT(m_pData->Background), "event", G_CALLBACK(on_event), m_pWidget);
 		g_signal_connect(G_OBJECT(m_pWidget), "destroy", G_CALLBACK(on_destroy), this);
 		g_signal_connect(G_OBJECT(m_pWidget), "size_allocate", G_CALLBACK(on_size), this);
+		g_signal_connect(G_OBJECT(m_pWidget), "realize", G_CALLBACK(gnome_canvas_gcp_update_bounds), this);
 		gtk_widget_show(m_pWidget);
 		m_Widgets.push_back(m_pWidget);
 		if (pWidget)
@@ -598,10 +601,12 @@ void gcpView::OnReceive(GtkClipboard* clipboard, GtkSelectionData* selection_dat
 	guint *DataType = (clipboard == gtk_clipboard_get(GDK_SELECTION_CLIPBOARD))? &ClipboardDataType: &ClipboardDataType1;
 	g_return_if_fail((selection_data->target == gdk_atom_intern (targets[*DataType].target, FALSE)));
 	if (pActiveTool->OnReceive(clipboard, selection_data, *DataType)) return;
-	else if (pActiveTool)
+//	if (ToolsDlg) ToolsDlg->Select(SelectId);
+	else if (pActiveTool/* && pActiveTool->Activate(false)*/)
 	{
 		pApp->ActivateTool("Select", true);
 		pActiveTool = pApp->GetActiveTool();
+//		if (pActiveTool) pActiveTool->Activate(true);
 	}
 	if (!pActiveTool || (pActiveTool != pApp->GetTool("Select"))) return;
 	xmlDocPtr xml;
@@ -770,18 +775,12 @@ bool gcpView::OnKeyRelease(GtkWidget* w, GdkEventKey* event)
 	return false;
 }
 	
-bool gcpView::OnSize(int width, int height)
+bool gcpView::OnSize (GtkWidget *w, int width, int height)
 {
-	if ((width == m_width) && (height == m_height)) return true;
-	std::list<GtkWidget*>::iterator i;
-	m_height = height;
-	m_width = width;
-	for (i = m_Widgets.begin(); i != m_Widgets.end(); i++)
-	{
-		gnome_canvas_set_scroll_region(GNOME_CANVAS(*i), 0, 0, width, height);
-		gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(*i), "data");
-		if (pData->Background) g_object_set(G_OBJECT(pData->Background), "x2", (double) width, "y2", (double) height, NULL);
-	}
+	gcpWidgetData* pData = (gcpWidgetData*) g_object_get_data (G_OBJECT (w), "data");
+	gnome_canvas_set_scroll_region (GNOME_CANVAS (w), 0, 0, (double) width / pData->Zoom, (double) height / pData->Zoom);
+	if (pData->Background)
+		g_object_set (G_OBJECT (pData->Background), "x2", (double) width / pData->Zoom, "y2", (double) height / pData->Zoom, NULL);
 	return true;
 }
 
@@ -790,9 +789,11 @@ void gcpView::UpdateSize(double x1, double y1, double x2, double y2)
 	if (x1 < 0.0) x2 -= x1;
 	if (y1 < 0.0) y2 -= y1;
 	list<GtkWidget*>::iterator i;
-	if ((x2 > m_width) || (y2 > m_height))
-		for (i = m_Widgets.begin(); i != m_Widgets.end(); i++)
-			gtk_widget_set_size_request(*i,(int)ceil(x2), (int)ceil(y2));
+	if ((x2 != m_width) || (y2 != m_height))
+		for (i = m_Widgets.begin(); i != m_Widgets.end(); i++) {
+			gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(*i), "data");
+			gtk_widget_set_size_request(*i,(int)ceil(x2 * pData->Zoom), (int)ceil(y2 * pData->Zoom));
+		}
 	if ((x1 < 0.0) || (y1 < 0.0))
 	{
 		x1 = - x1;
@@ -822,12 +823,8 @@ void gcpView::OnSelectAll()
 	gcpTool* pActiveTool = pApp->GetTool("Select");
 	if (pActiveTool) pApp->ActivateTool("Select", true);
 	m_pData->SelectAll();
-	if (pActiveTool) {
+	if (pActiveTool)
 		pActiveTool->AddSelection(m_pData);
-		pApp->ActivateToolItem ("Merge", ((m_pData->SelectedObjects.size () == 2) &&
-			(m_pData->SelectedObjects.front()->GetType() == MoleculeType) &&
-			(m_pData->SelectedObjects.back()->GetType() == MoleculeType)));
-	}
 }
 
 void gcpView::ExportImage (const char* filename, const char* type)
@@ -1017,7 +1014,7 @@ void gcpView::ExportImage (const char* filename, const char* type)
 		xmlNewDtd (doc,
 			   (const xmlChar*)"svg", (const xmlChar*)"-//W3C//DTD SVG 1.1//EN",
 			   (const xmlChar*)"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd");
-		doc->children = xmlNewDocNode (doc, NULL, (const xmlChar*)"svg", NULL);
+		xmlDocSetRootElement (doc, xmlNewDocNode (doc, NULL, (const xmlChar*)"svg", NULL));
 		xmlNsPtr ns = xmlNewNs (doc->children, (const xmlChar*)"http://www.w3.org/2000/svg", NULL);
 		xmlSetNs (doc->children, ns);
 		xmlNewProp (doc->children, (const xmlChar*)"version", (const xmlChar*)"1.1");
@@ -1083,5 +1080,13 @@ void gcpView::EnsureSize ()
 	while (canvas->idle_id)
 		gtk_main_iteration();
 	gnome_canvas_update_now (canvas);
-	g_signal_emit_by_name (m_pWidget, "update_bounds");
+	if (GTK_WIDGET_REALIZED (m_pWidget))
+		g_signal_emit_by_name (m_pWidget, "update_bounds");
+}
+
+void gcpView::Zoom (double zoom)
+{
+	m_pData->Zoom = zoom;
+	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), zoom);
+	EnsureSize ();
 }
